﻿using System;
using System.Web;
using System.Web.Security;
using Orchard.ContentManagement;
using Orchard.Logging;
using Orchard.Mvc;
using Orchard.Security;
using Orchard.Services;

namespace AlexZh.WindowsAuthentication.Services
{
	/// <summary>
	/// Copy of system FormsAuthenticationService
	/// </summary>
	public class FormsAuthenticationService
	{
		private readonly IClock clock;
		private readonly IContentManager contentManager;
		private readonly IHttpContextAccessor httpContextAccessor;
		private IUser signedInUser;

		public FormsAuthenticationService(IClock clock, IContentManager contentManager,
		                                  IHttpContextAccessor httpContextAccessor)
		{
			this.clock = clock;
			this.contentManager = contentManager;
			this.httpContextAccessor = httpContextAccessor;

			Logger = NullLogger.Instance;

			// TEMP: who can say...
			ExpirationTimeSpan = TimeSpan.FromHours(6);
		}

		public ILogger Logger { get; set; }

		public TimeSpan ExpirationTimeSpan { get; set; }

		#region IAuthenticationService Members

		public void SignIn(IUser user, bool createPersistentCookie)
		{
			DateTime now = clock.UtcNow.ToLocalTime();
			string userData = Convert.ToString(user.Id);

			var ticket = new FormsAuthenticationTicket(
				1 /*version*/,
				user.UserName,
				now,
				now.Add(ExpirationTimeSpan),
				createPersistentCookie,
				userData,
				FormsAuthentication.FormsCookiePath);

			string encryptedTicket = FormsAuthentication.Encrypt(ticket);

			var cookie = new HttpCookie(FormsAuthentication.FormsCookieName, encryptedTicket)
			             	{
			             		HttpOnly = true,
			             		Secure = FormsAuthentication.RequireSSL,
			             		Path = FormsAuthentication.FormsCookiePath
			             	};
			if (FormsAuthentication.CookieDomain != null)
			{
				cookie.Domain = FormsAuthentication.CookieDomain;
			}

			HttpContextBase httpContext = httpContextAccessor.Current();
			httpContext.Response.Cookies.Add(cookie);
			signedInUser = user;
		}

		public void SignOut()
		{
			signedInUser = null;
			FormsAuthentication.SignOut();
		}

		public void SetAuthenticatedUserForRequest(IUser user)
		{
			signedInUser = user;
		}

		public IUser GetAuthenticatedUser()
		{
			if (signedInUser != null)
				return signedInUser;

			HttpContextBase httpContext = httpContextAccessor.Current();
			if (httpContext == null || !httpContext.Request.IsAuthenticated || !(httpContext.User.Identity is FormsIdentity))
			{
				return null;
			}

			var formsIdentity = (FormsIdentity) httpContext.User.Identity;
			string userData = formsIdentity.Ticket.UserData;
			int userId;
			if (!int.TryParse(userData, out userId))
			{
				Logger.Fatal("User id not a parsable integer");
				return null;
			}
			return contentManager.Get(userId).As<IUser>();
		}

		#endregion
	}
}